package bk;

import animation.AlgorithmStage;
import animation.AnimatedAlgorithm;
import animation.AnimationController;
import animation.PseudoCodeNode;
import graph.LayeredGraphNode;

/**
 * The main stage of the BK node placement algorithm.
 * @author kolja
 *
 */
public class BKNodePlacement extends AnimatedAlgorithm {

	/*
	 * Private data structures to store the process of the algorithm
	 */
	
	private enum State
	{
		CONFLICTS,
		LAYOUT1,
		LAYOUT2,
		LAYOUT3,
		LAYOUT4,
		COMBINE
	}
	
	private ConflictDetection conftion;
	private State state;
	private ExtremalLayoutCalc layouts[];
	private Combine combine;
    private PseudoCodeNode conflictsNode; 
    private PseudoCodeNode layout1Node;
    private PseudoCodeNode layout2Node;
    private PseudoCodeNode layout3Node;
    private PseudoCodeNode layout4Node;
    private PseudoCodeNode combineNode;
	
	public BKNodePlacement(AnimationController controller, LayeredGraphNode graph) {
		super(controller, graph);
		state = State.CONFLICTS;
		conftion = new ConflictDetection( graph );
		layouts = new ExtremalLayoutCalc[ 4 ];
		layouts[ 0 ] = new ExtremalLayoutCalc( ExtremalLayoutCalc.LayoutType.TOP_BOTTOM_LEFT, graph );
		layouts[ 1 ] = new ExtremalLayoutCalc( ExtremalLayoutCalc.LayoutType.TOP_BOTTOM_RIGHT, graph );
		layouts[ 2 ] = new ExtremalLayoutCalc( ExtremalLayoutCalc.LayoutType.BOTTOM_TOP_LEFT, graph );
		layouts[ 3 ] = new ExtremalLayoutCalc( ExtremalLayoutCalc.LayoutType.BOTTOM_TOP_RIGHT, graph );
		combine = new Combine( graph );
	}

	@Override
	public StageStatus forwardStep() {
		switch( state )
		{
		case CONFLICTS:
		    conflictsNode.setSelected( true );
			if( conftion.forwardStep() == StageStatus.FINISHED )
			{
	            conflictsNode.setSelected( false );
	            layout1Node.setSelected( true );
				state = State.LAYOUT1;
			}
			break;
		case LAYOUT1:
			if( layouts[ 0 ].forwardStep() == StageStatus.FINISHED )
			{
	            layout1Node.setSelected( false );
	            layout2Node.setSelected( true );
				state = State.LAYOUT2;
			}
			break;
		case LAYOUT2:
			if( layouts[ 1 ].forwardStep() == StageStatus.FINISHED )
			{
	            layout2Node.setSelected( false );
                layout3Node.setSelected( true );
				state = State.LAYOUT3;
			}
			break;
		case LAYOUT3:
			if( layouts[ 2 ].forwardStep() == StageStatus.FINISHED )
			{
                layout3Node.setSelected( false );
                layout4Node.setSelected( true );
				state = State.LAYOUT4;
			}
			break;
		case LAYOUT4:
			if( layouts[ 3 ].forwardStep() == StageStatus.FINISHED )
			{
	            layout4Node.setSelected( false );
                combineNode.setSelected( true );
				state = State.COMBINE;
			}
			break;
		case COMBINE:
			return combine.forwardStep();
		}
		return StageStatus.UNFINISHED;
	}

	@Override
	public StageStatus backwardStep() {
		switch( state )
		{
		case CONFLICTS:
			return conftion.backwardStep();
		case LAYOUT1:
			if( layouts[ 0 ].backwardStep() == StageStatus.FINISHED )
			{
	            layout1Node.setSelected( false );
	            conflictsNode.setSelected( true );
				state = State.CONFLICTS;
			}
			break;
		case LAYOUT2:
			if( layouts[ 1 ].backwardStep() == AlgorithmStage.StageStatus.FINISHED )
			{
	            layout2Node.setSelected( false );
	            layout1Node.setSelected( true );
				state = State.LAYOUT1;
			}
			break;
		case LAYOUT3:
			if( layouts[ 2 ].backwardStep() == AlgorithmStage.StageStatus.FINISHED )
			{
	            layout3Node.setSelected( false );
	            layout2Node.setSelected( true );
				state = State.LAYOUT2;
			}
			break;
		case LAYOUT4:
			if( layouts[ 3 ].backwardStep() == AlgorithmStage.StageStatus.FINISHED )
			{
                layout4Node.setSelected( false );
                layout3Node.setSelected( true );
				state = State.LAYOUT3;
			}
			break;
		case COMBINE:
			if( combine.backwardStep() == AlgorithmStage.StageStatus.FINISHED )
			{
	            combineNode.setSelected( false );
	            layout4Node.setSelected( true );
				state = State.LAYOUT4;
			}
			break;
		}
		return StageStatus.UNFINISHED;
	}
	
	public PseudoCodeNode createPseudocodeTree()
	{
	    PseudoCodeNode root = new PseudoCodeNode( "BK Node Placement Algorithm" );
        conflictsNode = conftion.createPseudocodeTree();
        layout1Node = layouts[ 0 ].createPseudocodeTree();
        layout2Node = layouts[ 1 ].createPseudocodeTree();
        layout3Node = layouts[ 2 ].createPseudocodeTree();
        layout4Node = layouts[ 3 ].createPseudocodeTree();
        combineNode = combine.createPseudocodeTree();
        root.add( conflictsNode );
        root.add( layout1Node );
        root.add( layout2Node );
        root.add( layout3Node );
        root.add( layout4Node );
        root.add( combineNode );
        return root;
	}
}
